@@ -1,4 +1,5 @@
 """Implementation for deprecated-local-action rule."""
+
 # Copyright (c) 2016, Tsukinowa Inc. <info@tsukinowa.jp>
 # Copyright (c) 2018, Ansible Project
 from __future__ import annotations
@@ -43,10 +44,7 @@ class TaskNoLocalAction(AnsibleLintRule, TransformMixin):
     ) -> bool | str:
         """Return matches for a task."""
         raw_task = task["__raw_task__"]
-        if "local_action" in raw_task:
-            return True
-
-        return False
+        return "local_action" in raw_task
 
     def transform(
         self,
