@@ -1,10 +1,11 @@
 """Implementation of var-naming rule."""
+
 from __future__ import annotations
 
 import keyword
 import re
 import sys
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, NamedTuple
 
 from ansible.parsing.yaml.objects import AnsibleUnicode
 from ansible.vars.reserved import get_reserved_names
@@ -21,13 +22,20 @@ from ansiblelint.file_utils import Lintable
 from ansiblelint.rules import AnsibleLintRule, RulesCollection
 from ansiblelint.runner import Runner
 from ansiblelint.skip_utils import get_rule_skips_from_line
-from ansiblelint.text import has_jinja, is_fqcn_or_name
+from ansiblelint.text import has_jinja, is_fqcn, is_fqcn_or_name
 from ansiblelint.utils import parse_yaml_from_file
 
 if TYPE_CHECKING:
     from ansiblelint.utils import Task
 
 
+class Prefix(NamedTuple):
+    """Prefix."""
+
+    value: str = ""
+    from_fqcn: bool = False
+
+
 class VariableNamingRule(AnsibleLintRule):
     """All variables should be named using only lowercase and underscores."""
 
@@ -108,7 +116,8 @@ class VariableNamingRule(AnsibleLintRule):
         self,
         ident: str,
         *,
-        prefix: str = "",
+        prefix: Prefix | None = None,
+        file: Lintable,
     ) -> MatchError | None:
         """Return a MatchError if the variable name is not valid, otherwise None."""
         if not isinstance(ident, str):  # pragma: no cover
@@ -116,6 +125,7 @@ class VariableNamingRule(AnsibleLintRule):
                 tag="var-naming[non-string]",
                 message="Variables names must be strings.",
                 rule=self,
+                lintable=file,
             )
 
         if ident in ANNOTATION_KEYS or ident in self.allowed_special_names:
@@ -128,6 +138,7 @@ class VariableNamingRule(AnsibleLintRule):
                 tag="var-naming[non-ascii]",
                 message=f"Variables names must be ASCII. ({ident})",
                 rule=self,
+                lintable=file,
             )
 
         if keyword.iskeyword(ident):
@@ -135,6 +146,7 @@ class VariableNamingRule(AnsibleLintRule):
                 tag="var-naming[no-keyword]",
                 message=f"Variables names must not be Python keywords. ({ident})",
                 rule=self,
+                lintable=file,
             )
 
         if ident in self.reserved_names:
@@ -142,6 +154,7 @@ class VariableNamingRule(AnsibleLintRule):
                 tag="var-naming[no-reserved]",
                 message=f"Variables names must not be Ansible reserved names. ({ident})",
                 rule=self,
+                lintable=file,
             )
 
         if ident in self.read_only_names:
@@ -149,6 +162,7 @@ class VariableNamingRule(AnsibleLintRule):
                 tag="var-naming[read-only]",
                 message=f"This special variable is read-only. ({ident})",
                 rule=self,
+                lintable=file,
             )
 
         # We want to allow use of jinja2 templating for variable names
@@ -157,25 +171,30 @@ class VariableNamingRule(AnsibleLintRule):
                 tag="var-naming[no-jinja]",
                 message="Variables names must not contain jinja2 templating.",
                 rule=self,
+                lintable=file,
             )
 
-        if not bool(self.re_pattern.match(ident)):
+        if not bool(self.re_pattern.match(ident)) and (
+            not prefix or not prefix.from_fqcn
+        ):
             return MatchError(
                 tag="var-naming[pattern]",
                 message=f"Variables names should match {self.re_pattern_str} regex. ({ident})",
                 rule=self,
+                lintable=file,
             )
 
         if (
             prefix
-            and not ident.lstrip("_").startswith(f"{prefix}_")
-            and not has_jinja(prefix)
-            and is_fqcn_or_name(prefix)
+            and not ident.lstrip("_").startswith(f"{prefix.value}_")
+            and not has_jinja(prefix.value)
+            and is_fqcn_or_name(prefix.value)
         ):
             return MatchError(
                 tag="var-naming[no-role-prefix]",
-                message=f"Variables names from within roles should use {prefix}_ as a prefix.",
+                message=f"Variables names from within roles should use {prefix.value}_ as a prefix.",
                 rule=self,
+                lintable=file,
             )
         return None
 
@@ -189,9 +208,8 @@ class VariableNamingRule(AnsibleLintRule):
         # If the Play uses the 'vars' section to set variables
         our_vars = data.get("vars", {})
         for key in our_vars:
-            match_error = self.get_var_naming_matcherror(key)
+            match_error = self.get_var_naming_matcherror(key, file=file)
             if match_error:
-                match_error.filename = str(file.path)
                 match_error.lineno = (
                     key.ansible_pos[1]
                     if isinstance(key, AnsibleUnicode)
@@ -203,12 +221,15 @@ class VariableNamingRule(AnsibleLintRule):
             if isinstance(role, AnsibleUnicode):
                 continue
             role_fqcn = role.get("role", role.get("name"))
-            prefix = role_fqcn.split("/" if "/" in role_fqcn else ".")[-1]
+            prefix = self._parse_prefix(role_fqcn)
             for key in list(role.keys()):
                 if key not in PLAYBOOK_ROLE_KEYWORDS:
-                    match_error = self.get_var_naming_matcherror(key, prefix=prefix)
+                    match_error = self.get_var_naming_matcherror(
+                        key,
+                        prefix=prefix,
+                        file=file,
+                    )
                     if match_error:
-                        match_error.filename = str(file.path)
                         match_error.message += f" (vars: {key})"
                         match_error.lineno = (
                             key.ansible_pos[1]
@@ -219,9 +240,12 @@ class VariableNamingRule(AnsibleLintRule):
 
             our_vars = role.get("vars", {})
             for key in our_vars:
-                match_error = self.get_var_naming_matcherror(key, prefix=prefix)
+                match_error = self.get_var_naming_matcherror(
+                    key,
+                    prefix=prefix,
+                    file=file,
+                )
                 if match_error:
-                    match_error.filename = str(file.path)
                     match_error.message += f" (vars: {key})"
                     match_error.lineno = (
                         key.ansible_pos[1]
@@ -249,10 +273,9 @@ class VariableNamingRule(AnsibleLintRule):
     ) -> list[MatchError]:
         """Return matches for task based variables."""
         results = []
-        prefix = ""
-        filename = "" if file is None else str(file.path)
+        prefix = Prefix()
         if file and file.parent and file.parent.kind == "role":
-            prefix = file.parent.path.name
+            prefix = Prefix(file.parent.path.name)
         ansible_module = task["action"]["__ansible_module__"]
         # If the task uses the 'vars' section to set variables
         our_vars = task.get("vars", {})
@@ -260,13 +283,16 @@ class VariableNamingRule(AnsibleLintRule):
             action = task["action"]
             if isinstance(action, dict):
                 role_fqcn = action.get("name", "")
-                prefix = role_fqcn.split("/" if "/" in role_fqcn else ".")[-1]
+                prefix = self._parse_prefix(role_fqcn)
         else:
-            prefix = ""
+            prefix = Prefix()
         for key in our_vars:
-            match_error = self.get_var_naming_matcherror(key, prefix=prefix)
+            match_error = self.get_var_naming_matcherror(
+                key,
+                prefix=prefix,
+                file=file or Lintable(""),
+            )
             if match_error:
-                match_error.filename = filename
                 match_error.lineno = our_vars[LINE_NUMBER_KEY]
                 match_error.message += f" (vars: {key})"
                 results.append(match_error)
@@ -279,9 +305,12 @@ class VariableNamingRule(AnsibleLintRule):
                 and x != "cacheable",
                 task["action"].keys(),
             ):
-                match_error = self.get_var_naming_matcherror(key, prefix=prefix)
+                match_error = self.get_var_naming_matcherror(
+                    key,
+                    prefix=prefix,
+                    file=file or Lintable(""),
+                )
                 if match_error:
-                    match_error.filename = filename
                     match_error.lineno = task["action"][LINE_NUMBER_KEY]
                     match_error.message += f" (set_fact: {key})"
                     results.append(match_error)
@@ -289,10 +318,13 @@ class VariableNamingRule(AnsibleLintRule):
         # If the task registers a variable
         registered_var = task.get("register", None)
         if registered_var:
-            match_error = self.get_var_naming_matcherror(registered_var, prefix=prefix)
+            match_error = self.get_var_naming_matcherror(
+                registered_var,
+                prefix=prefix,
+                file=file or Lintable(""),
+            )
             if match_error:
                 match_error.message += f" (register: {registered_var})"
-                match_error.filename = filename
                 match_error.lineno = task[LINE_NUMBER_KEY]
                 results.append(match_error)
 
@@ -303,15 +335,17 @@ class VariableNamingRule(AnsibleLintRule):
         results: list[MatchError] = []
         raw_results: list[MatchError] = []
         meta_data: dict[AnsibleUnicode, Any] = {}
-        filename = "" if file is None else str(file.path)
 
         if str(file.kind) == "vars" and file.data:
             meta_data = parse_yaml_from_file(str(file.path))
             for key in meta_data:
-                prefix = file.role if file.role else ""
-                match_error = self.get_var_naming_matcherror(key, prefix=prefix)
+                prefix = Prefix(file.role) if file.role else Prefix()
+                match_error = self.get_var_naming_matcherror(
+                    key,
+                    prefix=prefix,
+                    file=file,
+                )
                 if match_error:
-                    match_error.filename = filename
                     match_error.lineno = key.ansible_pos[1]
                     match_error.message += f" (vars: {key})"
                     raw_results.append(match_error)
@@ -329,6 +363,9 @@ class VariableNamingRule(AnsibleLintRule):
             results.extend(super().matchyaml(file))
         return results
 
+    def _parse_prefix(self, fqcn: str) -> Prefix:
+        return Prefix("" if "." in fqcn else fqcn.split("/")[-1], is_fqcn(fqcn))
+
 
 # testing code to be loaded only with pytest or when executed the rule file
 if "pytest" in sys.modules:
@@ -388,6 +425,25 @@ if "pytest" in sys.modules:
             assert result.tag == expected_errors[idx][0]
             assert result.lineno == expected_errors[idx][1]
 
+    def test_invalid_vars_diff_files(
+        default_rules_collection: RulesCollection,
+    ) -> None:
+        """Test rule matches."""
+        results = Runner(
+            Lintable("examples/playbooks/vars/rule_var_naming_fails_files"),
+            rules=default_rules_collection,
+        ).run()
+        expected_errors = (
+            ("var-naming[pattern]", 2),
+            ("var-naming[pattern]", 3),
+            ("var-naming[pattern]", 2),
+            ("var-naming[pattern]", 3),
+        )
+        assert len(results) == len(expected_errors)
+        for idx, result in enumerate(results):
+            assert result.tag == expected_errors[idx][0]
+            assert result.lineno == expected_errors[idx][1]
+
     def test_var_naming_with_role_prefix(
         default_rules_collection: RulesCollection,
     ) -> None:
@@ -433,6 +489,17 @@ if "pytest" in sys.modules:
         assert result.returncode == RC.SUCCESS
         assert "var-naming" not in result.stdout
 
+    def test_var_naming_with_pattern_foreign_role() -> None:
+        """Test rule matches."""
+        role_path = "examples/playbooks/bug-4095.yml"
+        conf_path = "examples/roles/var_naming_pattern/.ansible-lint"
+        result = run_ansible_lint(
+            f"--config-file={conf_path}",
+            role_path,
+        )
+        assert result.returncode == RC.SUCCESS
+        assert "var-naming" not in result.stdout
+
     def test_var_naming_with_include_tasks_and_vars() -> None:
         """Test with include tasks and vars."""
         role_path = "examples/roles/var_naming_pattern/tasks/include_task_with_vars.yml"
